/*
 * xzx    : Copyright (c) Des Herriott 1993, 1994
 * xcoupe : Copyright (c) Allan Skillman 1995.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Allan Skillman
 */

/* 
 * Source file fdi.h public interface to the floppy disk interface emulation
 */

#ifndef _FDI_H
#define _FDI_H

#define MAX_NO_SIDES	2
#define MAX_NO_TRACKS	80
#define MAX_NO_SECTORS	10

#define SECTOR_SIZE	512

/* The various disk format image sizes */

#define SAM_FORMAT_SIZE		819200
#define MSDOS_FORMAT_SIZE	737280

/* The VL1772 commands */
#define RESTORE		0x00
#define SEEK		0x10
#define STEP_NUPD	0x20
#define STEP_UPD	0x30
#define STEP_IN_NUPD	0x40
#define STEP_IN_UPD	0x50
#define STEP_OUT_NUPD	0x60
#define STEP_OUT_UPD	0x70
#define READ_1SECTOR	0x80
#define READ_MSECTOR	0x90
#define WRITE_1SECTOR	0xa0
#define WRITE_MSECTOR	0xb0
#define READ_ADDRESS	0xc0
#define READ_TRACK	0xe0
#define WRITE_TRACK	0xf0
#define FORCE_INTERRUPT	0xd0

/* The possible status bits */
#define BUSY			0x01
#define DRQ			0x02
#define LOST_DATA		0x04
#define CRC_ERROR		0x08
#define RECORD_NOT_FOUND	0x10
#define SPIN_UP			0x20
#define WRITE_PROTECT		0x40
#define MOTOR_ON		0x80


void fdi_out(uns16 port, uns8 byte);
uns8 fdi_in (uns16 port);

void insert_disk(char *fn, int disk_no);
void eject_disk(int disk_no);

#endif
