/*
 * xzx    : Copyright (c) Des Herriott 1993, 1994
 * xcoupe : Copyright (c) Allan Skillman 1995.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Allan Skillman (Based on xzx coding by Des Herriott)
 */

/*
 * Source file mem.c - where memory is actually held.
 *
 * The SAM memory is implemented in NMAINPAGES 16K pages and pointed to 
 * by the array *realMem[]. (The actual space is allocated at run time)
 * The Z80 Address space is asigned using an array of 4 pointers to this
 * space. The page number asigned to Sections A..D of the address space
 * is stored in real_pagenos[].
 * All memory addressing in the Z80 emulation is done indirectly via these
 * pointers - see file mem.h
*/

#include <stdlib.h>
#include <stdio.h>

#include "z80.h"
#include "mem.h"
#include "util.h"
#include "rtcfg.h"


uns8 *theMemory[4];		/* 4 pointers to real memory */
int real_pagenos[4];		/* index to real memory page */
uns8 *realMem[TOTAL_NO_PAGES];	/* real memory is kept here */


/* Allocate NMAINPAGES chunks of main memory, and the required
   number of external memory pages, each of size XCOUPE_PAGESIZE */
void
init_memory(void)
{
	static int inited = 0;
	int i, iMeg, NoMbs;

	if (inited) {
		xcoupe_mesg(XCOUPE_ERR, "attempt to re-initialise memory (failed)");
		return;
	}

	/* initialise the main SAM memory */
	for (i = 0; i < NMAINPAGES; i++) {
		if ((realMem[i] = (uns8 *)malloc(XCOUPE_PAGESIZE)) == (uns8 *)0)
			xcoupe_mesg(XCOUPE_FATAL, "can't allocate 16K for page %d!\n", i);
	}

	/* initialise the memory for the 2 roms */
	for (i = 0; i < 2; i++) {
		if ((realMem[ROM0+i] = (uns8 *)malloc(XCOUPE_PAGESIZE)) == (uns8 *)0)
			xcoupe_mesg(XCOUPE_FATAL, "can't allocate 16K for ROM %d!\n", i);
	}


	/* initialise the external memory if required*/
	if (NoMbs = GetCfg(externalMem)) {

		if (NoMbs > MAXEXPAGES) NoMbs = MAXEXPAGES;
		for (iMeg = 0; iMeg < NoMbs; iMeg++) {
			for (i = 0; i < NEXPAGES; i++) {
		
				if ((realMem[NMAINPAGES+i+(3-iMeg)*NEXPAGES] = (uns8 *)malloc(XCOUPE_PAGESIZE)) == (uns8 *)0)
					xcoupe_mesg(XCOUPE_FATAL, "can't allocate 16K for external page %d!\n", i+iMeg*NEXPAGES);
			}
		}
	}
}


/* Page in real memory page at <frame>, where <frame> is in range 0..3 */
void
page_in(int frame, int rpage)
{

	theMemory[frame] = realMem[rpage];
	real_pagenos[frame] = rpage;
/*	printf("page_in(%d,%d)\n",frame,rpage); */
}

/* Set up the initial memory pages; ROM0 paged in at 0x0000
 */
void
init_pages(void)
{
	page_in(0, ROM0); 
	page_in(1, 1);
	page_in(2, 2); 
	page_in(3, 3);
}
