/*
 * xzx    : Copyright (c) Des Herriott 1993, 1994
 * xcoupe : Copyright (c) Allan Skillman 1995.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Allan Skillman (Based on xzx coding by Des Herriott)
 */

/*
 * Header file mem.h - contains definitions, macros and routines 
 * for accessing the segmented memory model.
 */

#ifndef _XCOUPE_MEM_H
#define _XCOUPE_MEM_H

#include "xsam.h"
#include "rtcfg.h"


#if !defined(__GNUC__)
#define inline
#endif


/* 16K memory pages */
#define XCOUPE_PAGESIZE 16384

/* Constants related to memory organisation */

#define NMAINPAGES	32	/* Number of pages in the SAM's main memory (512K) */
#define NEXPAGES	64	/* Number of pages in 1Mb of external memory */ 
#define MAXEXPAGES	4       /* Maximum number of 1Mb external memory allowed */

/* The Roms always go at the end of the memory */

#define ROM0   		NMAINPAGES+(NEXPAGES*MAXEXPAGES)
#define ROM1   		ROM0 + 1
#define TOTAL_NO_PAGES	ROM1 + 1


extern uns8 *theMemory[4];
extern int real_pagenos[4];
extern uns8 *realMem[TOTAL_NO_PAGES];

extern void init_pages(void);

/* Map a 16 bit address through the memory indirection - allows fast paging */
#define VPAGE(addr) ((addr) >> 14)
#define RPAGE(addr) (real_pagenos[VPAGE(addr)])
#define MEM(addr) (theMemory[VPAGE(addr)][(addr) & 0x3fff])
#define READ_ONLY(addr) ((RPAGE(addr) >= ROM0) || (!((addr) & 0xc000) && LMPR_WPRAM))

void init_memory(void);
void page_in(int frame, int rpage);


/*
 * I know putting blocks of code in header files isn't very nice, but
 * this does let me take advantage of gcc's inlining...

 */
static inline uns8
read_byte(uns16 addr)
{
	return MEM(addr);
}

static inline uns16
read_word(uns16 addr)
{
	   return ((MEM(addr)) + (MEM((uns16)(addr + 1)) << 8));
}


#ifdef 0
#define read_byte(addr)	 MEM(addr)
#define read_word(addr) (MEM(addr)) + (MEM((uns16)((addr) + 1)) << 8)
#endif

static void
write_to_screen_vmpr0(uns16 addr, uns8 val) {

	int rpage = RPAGE(addr);

	/* from xsam.c */
	extern void (*screen_write)(uns16, uns8);

	addr &= 0x3fff;

        switch (VMPR_SCREEN_MODE) {
           
	        case MODE_1:      
		   if (addr < 6144)
			screen_write(addr, val);
		   else if (addr < 6912)
			attr_write_mode_1(addr, val);
		   break;
	        case MODE_2:      
		   if (addr < 6144)
			screen_write(addr, val);
		   else if ((addr & 0x2000) &&
		            (addr < 14336))
			attr_write_mode_2(addr, val);
		   break;
	        case MODE_3:
	        case MODE_4:      
		   screen_write(addr, val);
		   break;
	} /* End Switch */
	   
}  

static  void
write_to_screen_vmpr1(uns16 addr, uns8 val) {

	/* from xsam.c */
	extern void (*screen_write)(uns16, uns8);

	addr &= 0x3fff;
	if (!(addr & 0x2000)) 
		screen_write(0x4000 + addr, val);
        
}

static void 
write_byte(uns16 addr, uns8 val)
{

	if (READ_ONLY(addr)) 
		return;
	else {
		MEM(addr) = val;
		
		/* Only Do a screen write if required */
		if (RPAGE(addr) == VMPR_SCREEN_PAGE)
	 		write_to_screen_vmpr0(addr, val);
	 	else if ((VMPR_SCREEN_MODE > MODE_2)  &&
		         (RPAGE(addr) == (VMPR_SCREEN_PAGE + 1) & 0x1f))
		    	write_to_screen_vmpr1(addr, val);
	}
}




static void
write_word(uns16 addr, uns16 val)
{
#if 0
	write_byte(addr, val & 0xff);
	write_byte((uns16)(addr + 1), (val & 0xff00) >> 8);
#endif	
	if (!READ_ONLY(addr)) {
		MEM(addr) = val & 0xff;
		
		/* Only Do a screen write if required */
		if (RPAGE(addr) == VMPR_SCREEN_PAGE)
	 		write_to_screen_vmpr0(addr, val & 0xff);
	 	else if ((VMPR_SCREEN_MODE > MODE_2)  &&
		         (RPAGE(addr) == (VMPR_SCREEN_PAGE + 1) & 0x1f))
		    	write_to_screen_vmpr1(addr, val & 0xff);
	}
	addr++;
	if (!READ_ONLY(addr)) {
		MEM(addr) = (val & 0xff00) >> 8;
		
		/* Only Do a screen write if required */
		if (RPAGE(addr) == VMPR_SCREEN_PAGE)
		    	write_to_screen_vmpr0(addr, (val & 0xff00) >> 8);
	 	else if ((VMPR_SCREEN_MODE > MODE_2)  &&
		         (RPAGE(addr) == (VMPR_SCREEN_PAGE + 1) & 0x1f))
		    	write_to_screen_vmpr1(addr, (val & 0xff00) >> 8);
		
	}

}

#endif
