/*
 * xzx    : Copyright (c) Des Herriott 1993, 1994
 * xcoupe : Copyright (c) Allan Skillman 1995.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Allan Skillman (Based on xzx coding by Des Herriott)
 */

/* Header file rtcfg.h - for reading & setting run-time configuration */

#ifndef _XCOUPE_RTCFG_H
#define _XCOUPE_RTCFG_H

#include "string.h"

#define DEF_FONT "fixed"

struct cfg_block {
	char geometry[256];	/* X11 geometry spec. (size is ignored) */
	char font[256];		/* X11 font name */
	char visual[16];	/* X11 Visual name */
	int use_mitshm;		/* attempt to use MIT-SHM for image transfer */
	int priv_cmap;		/* use a private colormap */
#if 0
	int monochrome;		/* use only BlackPixel and WhitePixel */
#endif
	int recalibrate;	/* attempt to keep the interrupt rate at 50/sec */
	char rom0[256];		/* SAM ROM 0 image filename */
	char rom1[256];		/* SAM ROM 1 image filename */
	char lib_dir[1024];	/* directory to check for support files */
	char disk_1[256];	/* floppy disk image in drive 1 */
	char disk_2[256];	/* floppy disk image in drive 2 */
	int ifreq;		/* how often interrupts arrive (counter-style only) */
	int flashing;		/* is flashing enabled? */
	int rr_shm;		/* refresh frequency when using MITSHM */
	int rr_noshm;		/* as above, but not using MITSHM */
#ifdef SLOWDOWN
	int slowdown;		/* slowdown factor for very fast machines */
#endif
#ifdef JOY
	int use_js;		/* try to use a joystick */
	int js_tolerance;	/* joystick displacement tolerance */
	char js_device[256];	/* joystick device name (/dev/js[0|1]) */
#endif
#if defined(DEV_AUDIO) || defined(PCSPKR_AUDIO) || defined(NAS_AUDIO)
	int use_sound;		/* use Sparc, PC or NAS audio to emulate the speaker */
#ifdef DEV_AUDIO
	char audio_dev[256];	/* name of audio device */
#endif
#ifdef NAS_AUDIO
	char au_serv[256];	/* name of audio server */
#endif
#endif
#ifdef XBELL_AUDIO
	int bell_vol;		/* bell volume as a percentage */
#endif
	int externalMem;	/* No. Mb of external memory */
	int NoBorder;		/* Turn off SAM border simulation */
};

extern struct cfg_block cblock;

#define GetCfg(field) cblock.field
#define SetCfg(field,value) cblock.field = value
#define SetCfgStr(field,value) strcpy(cblock.field,value)

#endif

