/*
 * xzx    : Copyright (c) Des Herriott 1993, 1994
 * xcoupe : Copyright (c) Allan Skillman 1995.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Allan Skillman (Based on xzx coding by Des Herriott)
 */

/*
 * Header file xsam.h - public interface to sam and X11 code.
 */

#ifndef _XCOUPE_H
#define _XCOUPE_H

void start_emulator(int *argcp, char **argvp);
void attr_write_mode_1(uns16 addr, uns8 val);
void attr_write_mode_2(uns16 addr, uns8 val);
void init_ports(void);
void out_byte(uns16 port, uns8 val);
uns8 in_byte(uns16 port);
void do_flashing(void);
void force_screen_refresh(void);
void update_line(void);
void update_to_first_line(void);
void update_border_to_first_line(void);
void check_events(void);

char *dialog_event_loop(char *label);

/* a list of actions which can be bound to keypresses */
enum {
	KM_JOY_UP, KM_JOY_DOWN,
	KM_JOY_LEFT, KM_JOY_RIGHT,
	KM_JOY_FIRE,
#ifdef SLOWDOWN
	KM_KEY_FASTER, KM_KEY_SLOWER,
#endif
	NUM_KEYMAPPINGS
};


#define BORDER_WIDTH 	30
#define HSIZE 		256
#define VSIZE 		192

#ifdef PSEUDO_IO
#define PSEUDO_IO_LOWBYTE 0xbf
#define PSEUDO_STDIN  (0x0000 | PSEUDO_IO_LOWBYTE)
#define PSEUDO_STDOUT (0x0100 | PSEUDO_IO_LOWBYTE)
#define PSEUDO_STDERR (0x0200 | PSEUDO_IO_LOWBYTE)
#endif

/* Macros for access to sam ports */

#define SOUND_ADDR	511
#define SOUND_DATA	252

#define VMPR_PORT	252
#define HMPR_PORT	251
#define LMPR_PORT	250
#define MIDI_PORT	253
#define BORDER_PORT	254
#define LINE_PORT	249
#define CLUT_BASE_PORT	248
#define ATTR_PORT	255
#define KEYBOARD_PORT	254
#define STATUS_PORT	249
#define LPEN_PORT	248
#define HPEN_PORT	504
#define CLOCK_PORT	239

#define LEPR_PORT	128
#define HEPR_PORT	129

#define DISK1_PORT	224
#define DISK2_PORT	240

#define VMPR_SCREEN_PAGE	(vmpr & 0x1f)
#define VMPR_SCREEN_MODE        (vmpr & 0x60)
#define HMPR_PAGE		(hmpr & 0x1f)
#define HMPR_MD3COL	       ((hmpr & 0x60) >> 3)
#define HMPR_MCNTRL		(hmpr & 0x80)
#define LMPR_PAGE		(lmpr & 0x1f)
#define LMPR_RAM0		(lmpr & 0x20)
#define LMPR_ROM1		(lmpr & 0x40)
#define LMPR_WPRAM		(lmpr & 0x80)

#define BORD_VAL(x)	       ((((x) & 0x20 ) >> 2) | ((x) & 0x07))
#define BORD_SOFF		(border & 0x80)

#define MD3_SWITCH(x)	       (((x) == 0x01 || (x) == 0x02) ? (x) ^ 0x03 : (x))

#define MODE_1			0x00
#define MODE_2			0x20
#define MODE_3			0x40
#define MODE_4			0x60

/* The sam ports */

/* Read / Write ports */

extern uns8 vmpr;
extern uns8 hmpr;
extern uns8 lmpr;
extern uns8 midi;
extern uns8 border;

extern uns8 lepr;
extern uns8 hepr;

extern uns8 clock_reg;

/* Write only ports */

extern uns8 line_int;
extern uns8 clut[];

/* Read only ports */

extern uns8 attr_reg;
extern uns8 keyboard;
extern uns8 status_reg;
extern uns8 lpen;
extern uns8 hpen;

#endif
